@@ -131,6 +131,47 @@ def open_revision_doc(request, submission_id, version):
     return redirect(f"/document/{rev.document.id}/", permanent=True)
 
 
+# Check if the revision doc exists.
+@csrf_exempt
+@require_GET
+def check_revision_doc(request, submission_id, version):
+    api_key = request.GET.get("key")
+    submission = models.Submission.objects.get(id=submission_id)
+    journal_key = submission.journal.ojs_key
+    journal_id = submission.journal_id
+    res = 0
+
+    # Validate api key
+    if journal_key == api_key:
+        user_id = request.GET.get("user_id")
+        is_editor = request.GET.get("is_editor")
+
+        # Validate is_editor
+        try:
+            is_editor = int(is_editor)
+        except ValueError:
+            is_editor = 0
+
+        user = find_user(
+            journal_id, submission_id, version, user_id, is_editor
+        )
+
+        # Validate if user exists
+        if not user:
+            return HttpResponse("User not accessible", status=403)
+
+        # Validate if doc exists
+        try:
+            models.SubmissionRevision.objects.get(
+                submission_id=submission_id, version=version
+            )
+            res = 1
+        except Exception:
+            res = 0
+
+    return HttpResponse(res, status=200)
+
+
 # Send basic information about the current document and the journals that can
 # be submitted to. This information is used as a starting point to decide what
 # OJS-related UI elements to add on the editor page.
@@ -161,7 +202,8 @@ def get_doc_info(request):
         ).first()
         if revision:
             user_role = ""
-            if revision.reviewer_set.filter(user=request.user).count() > 0:
+            reviewer = revision.reviewer_set.filter(user=request.user).first()
+            if reviewer:
                 user_role = "reviewer"
             elif (
                 revision.submission.author_set.filter(
@@ -189,6 +231,10 @@ def get_doc_info(request):
                 "journal_id"
             ] = revision.submission.journal_id
             response["submission"]["user_role"] = user_role
+            if reviewer and reviewer.method == "doubleanonymous":
+                response["submission"]["contributors"] = {}
+            else:
+                response["submission"]["contributors"] = revision.contributors
         else:
             response["submission"]["status"] = "unsubmitted"
     journals = []
@@ -202,6 +248,7 @@ def get_doc_info(request):
             }
         )
     response["journals"] = journals
+
     status = 200
     return JsonResponse(response, status=status)
 
@@ -288,6 +335,22 @@ def accept_reviewer(request, submission_id, version):
         response["error"] = "Unknown reviewer"
         status = 403
         return JsonResponse(response, status=status)
+    review_method = request.POST.get("review_method")
+
+    # ojs-fiduswriter < 3.0.0.0 specified "access_rights" instead of "review_method".
+    if not review_method:
+        if request.POST.get("access_rights") == "comment":
+            review_method = "open"
+        else:
+            review_method = "doubleanonymous"
+
+    reviewer.method = review_method
+    reviewer.save()
+
+    if review_method == "open":
+        rights = "comment"
+    else:
+        rights = "review"
     # Make sure the connect document has reviewer access rights set for the
     # user.
     access_right = AccessRight.objects.filter(
@@ -295,12 +358,12 @@ def accept_reviewer(request, submission_id, version):
     ).first()
     if access_right is None:
         access_right = AccessRight(
-            document=revision.document, holder_obj=reviewer.user
+            document=revision.document,
+            holder_obj=reviewer.user,
+            path=revision.document.path,
         )
         status = 201
-    rights = "review"
-    if request.POST.get("access_rights") == "comment":
-        rights = "comment"
+
     access_right.rights = rights
     access_right.save()
     return JsonResponse(response, status=status)
@@ -347,7 +410,9 @@ def add_reviewer(request, submission_id, version):
     ).first()
     if access_right is None:
         access_right = AccessRight(
-            document=revision.document, holder_obj=reviewer.user
+            document=revision.document,
+            holder_obj=reviewer.user,
+            path=revision.document.path,
         )
         status = 201
     access_right.rights = "read-without-comments"
@@ -395,7 +460,13 @@ def create_copy(request, submission_id):
     status = 201
     api_key = request.POST.get("key")
     old_version = request.POST.get("old_version")
+    old_version_parts = old_version.split(".")
+    old_version_stage = int(old_version_parts[0])
+
     new_version = request.POST.get("new_version")
+    new_version_parts = new_version.split(".")
+    new_version_stage = int(new_version_parts[0])
+
     revision = models.SubmissionRevision.objects.get(
         submission_id=submission_id, version=old_version
     )
@@ -406,23 +477,12 @@ def create_copy(request, submission_id):
         status = 403
         return JsonResponse(response, status=status)
 
-    # Copy the document
-    document = helpers.copy_doc(
-        revision.document,
-        revision.submission.journal.editor,
-        submission_id,
-        new_version,
+    # Copy the revision
+    revision = helpers.copy_revision(
+        revision, old_version_stage, new_version_stage, new_version
     )
 
-    # Copy revision
-    revision.pk = None
-    revision.document = document
-    revision.version = new_version
-    revision.save()
-
     # Add user rights
-    new_version_parts = new_version.split(".")
-    new_version_stage = int(new_version_parts[0])
 
     # Rights for editors
     granted_user_ids = request.POST.get("granted_users").split(",")
@@ -436,8 +496,9 @@ def create_copy(request, submission_id):
                         new_version_stage
                     ]
                     AccessRight.objects.create(
-                        document=document,
+                        document=revision.document,
                         holder_obj=editor.user,
+                        path=revision.document.path,
                         rights=rights,
                     )
 
@@ -452,7 +513,10 @@ def create_copy(request, submission_id):
         authors = models.Author.objects.filter(submission=revision.submission)
         for author in authors:
             AccessRight.objects.create(
-                document=document, holder_obj=author.user, rights=access_right
+                document=revision.document,
+                holder_obj=author.user,
+                path=revision.document.path,
+                rights=access_right,
             )
 
     return JsonResponse(response, status=status)
@@ -512,7 +576,9 @@ def add_editor(request, submission_id):
                             int(stage_id)
                         ]
                         access_right = AccessRight(
-                            document=revision.document, holder_obj=editor.user
+                            document=revision.document,
+                            holder_obj=editor.user,
+                            path=revision.document.path,
                         )
                         access_right.rights = rights
                         access_right.save()
@@ -602,19 +668,29 @@ def add_author(request, submission_id):
         for revision in revisions:
             version = revision.version.split(".")
             stage_id = version[0]
-            if stage_id in ["1", "4"]:
+            if (
+                stage_id == "1"
+                or stage_id == "4"
+                or (stage_id == "3" and version[2] == "5")
+            ):
                 access_right = AccessRight.objects.filter(
                     document=revision.document, user=author.user
                 ).first()
+
                 if access_right is None:
                     access_right = AccessRight(
-                        document=revision.document, holder_obj=author.user
-                    )
-                    access_right.rights = (
-                        "read-without-comments"
-                        if stage_id == "1"
-                        else "write-tracked"
+                        document=revision.document,
+                        holder_obj=author.user,
+                        path=revision.document.path,
                     )
+
+                    if stage_id == "1":
+                        access_right.rights = "read-without-comments"
+                    elif stage_id == "3":
+                        access_right.rights = "write"
+                    else:
+                        access_right.rights = "write-tracked"
+
                     access_right.save()
                     status = 201
 
